# Project information

project = "GooseHDF5"
copyright = "2017-2022, Tom de Geus"
author = "Tom de Geus"

# General configuration

extensions = ["sphinx.ext.imgmath", "sphinx.ext.autodoc", "sphinx.ext.autosummary", "sphinxarg.ext"]

exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# Options for HTML output

html_theme = "sphinx_rtd_theme"
